#########################################################################
# SopcastSports
# Daniel Bardsley
# daniel@bardsley.org.uk
# created: 17/12/07
#########################################################################
# gui_events.tcl
# defines various GUI functionality including procedures to update
# information displays, create new dialogs and all things GUI related.
#########################################################################

# play button has been pressed. stream_no is the stream to play
# uses mplayer to connect to sp-sc which should be running
# mplayer execution will fail if stream is not started
proc push_play {stream_no} {
	set port [expr "($stream_no*100)+9000"]
	set vport [expr "$port + 1"]
	exec mplayer -vo x11 http://127.0.0.1:$vport -msglevel all=-1 -zoom &
}

# reload the game data from myp2p.eu and repopulate list boxes
# will fail (hard) if loaddata fails
proc push_reload {} {
	loaddata
	global .lst_games
	global .lst_streams
	.lst_games delete 0 [.lst_games size]
	.lst_streams delete 0 [.lst_streams size]
	.lab_streams configure -text "Sopcast Streams"
	buildguidata
}

# a game has been double clicked: find the streams associated with a given game and display
# in the lower listbox (lst.streams)
proc update_game {} {
	global .lst_games
	global .lst_streams
	global livegameslinks
	
	.lst_streams delete 0 [.lst_streams size]
	.lab_streams configure -text [.lst_games get [.lst_games curselection]]
	
	# search livegamelinks for links associated with the currently selected game.
	# Should be refactored to include the case when position of games in the top list box
	# is not associated with their gameid. ie. we should not use [.lst_games curselection]
	# to get the selected gameid - we should actually search livegames for the selected string 
	set nLinks [expr "[array size livegameslinks]/3"]
	for {set i 0} {$i < $nLinks} {incr i} {
		if { $livegameslinks($i,gameid) == [.lst_games curselection] } {
			.lst_streams insert end $livegameslinks($i,link)
		}
	}
}

# display a dialog to allow the user to enter a sopcast stream url manually
proc open_stream_clicked { } {
	if {![winfo exists .opendlg]} {
		# open streams window
		toplevel .opendlg
		wm title .opendlg "Open Stream"
		wm resizable .opendlg FALSE FALSE
		
		entry .opendlg.ent -width 30
		button .opendlg.loadstream -text "Load" -command "open_stream_clicked_helper"
		
		grid .opendlg.ent -row 1 -column 1
		grid .opendlg.loadstream -row 1 -column 2
	}
}

# helper function to get manually typed sopcast stream and fire load_stream
proc open_stream_clicked_helper {} {
	set sopcastlink [.opendlg.ent get]
	load_stream $sopcastlink
}

# Add the loaded games into the GUI
proc buildguidata {} {
	global livegames
	set nGames [expr "[array size livegames]/2"]
	for {set i 0} {$i < $nGames} {incr i} {
		.lst_games insert end $livegames($i,game)
	}
}

# load and parse the data from myp2p.eu
# Fill livegames and livegameslinks with data
proc loaddata { } {
	global livegames
	global livegameslinks
	global currentgame_id
	
	unset livegames
	unset livegameslinks
	set livegames(ID) -3; #ignore the first two links as these are not events
	set livegameslinks(ID) -1
		
	# Get list of live games
	exec rm tmp/* -f
	
	if { [catch { exec wget http://www.myp2p.eu/feeds/nowplaying.xml -q -O tmp/nowplaying.xml } msg] } {
		puts "wget had errors getting www.myp2p.eu/feeds/nowplaying.xml"
		puts "Error was: $::errorInfo"
		exit
	}
		
	set livegamesraw [readfile "tmp/nowplaying.xml"]
	# we should really check the load worked here
	
	# parse live games and links
	htmlparse::parse -cmd parseNowPlaying $livegamesraw

	unset livegames(-2,game)
	unset livegames(-2,link)
	unset livegames(-1,game)
	unset livegames(-1,link)

	set nGames [expr "[array size livegames]/2"]
	for {set i 0} {$i < $nGames} {incr i} {
		puts "$livegames($i,game)"

		# load and parse the link for this game
		if { [catch { exec wget $livegames($i,link) -O tmp/game.html -q } msg] } {
			puts "wget had errors getting $livegames($i,link):"
			puts "Error was: $::errorInfo"
			exit
		}
			
		set gameraw [readfile "tmp/game.html"]
		exec rm tmp/game.html -f
		
		set currentgame_id $i
		htmlparse::parse -cmd parseGame $gameraw	
	}	
}

# updates the main sopcastsports GUI
# method is called to update stream buffer levels, open channel names etc.
proc update_gui {stream_no} {
	global openchannels
	
	set bufferlevel $openchannels($stream_no,buffer)
	set up [expr "$openchannels($stream_no,up)/1000"]
	set down [expr "$openchannels($stream_no,down)/1000"]
	set nPeers $openchannels($stream_no,nPeers)
	set game $openchannels($stream_no,game)
	set stream $openchannels($stream_no,stream)
	set uptext "upload: $up kb/sec"
	set downtext "download: $down kb/sec"
	set peertext "peers: $nPeers"
	
	# update main progress bar
	if {[winfo exists .streams]} {
		.streams.prog$stream_no configure -value [expr "double ($bufferlevel)/100"]
		.streams.prog$stream_no configure -text $stream
	}
	
	# update stats window if it's open
	if {[winfo exists .infostream$stream_no]} {
		.infostream$stream_no.game configure -text $game
		.infostream$stream_no.link configure -text $stream
		.infostream$stream_no.up configure -text $uptext
		.infostream$stream_no.down configure -text $downtext
		.infostream$stream_no.peers configure -text $peertext
	}
}

# opens the stream info dialog for the given stream number
proc view_stream {stream_no} {
	global openchannels
	
	if {![winfo exists .infostream$stream_no]} {
		toplevel .infostream$stream_no
		wm resizable .infostream$stream_no FALSE FALSE
		wm title .infostream$stream_no "Stream $stream_no Stats"
	
		label .infostream$stream_no.game -text "Statistics Unavaliable"
		label .infostream$stream_no.link -text "Statistics Unavaliable"
		label .infostream$stream_no.up -text "Statistics Unavaliable"
		label .infostream$stream_no.down -text "Statistics Unavaliable"
		label .infostream$stream_no.peers -text "Statistics Unavaliable"
				
		pack .infostream$stream_no.game -in .infostream$stream_no
		pack .infostream$stream_no.link -in .infostream$stream_no
		pack .infostream$stream_no.up -in .infostream$stream_no
		pack .infostream$stream_no.down -in .infostream$stream_no
		pack .infostream$stream_no.peers -in .infostream$stream_no
	}
	wm deiconify .infostream$stream_no
}

# load the passed sopcast stream using sp-sc
# then add the stream data to the openchannels associative array
# also create a fileevent callback to process sp-sc output.
proc load_stream {sopcastlink} {
	global .lst_streams
	global livegames
	global livegameslinks
	global openchannels
	global freechannels
	
	if {![winfo exists .streams]} {
		show_streams
	}
	
	# find next free stream
	if { [llength $freechannels] == 0 } {
		puts "No free slots"
	} else {
		#set sopcastlink [.lst_streams get [.lst_streams curselection]]
		
		# find sopcastlink in livegamelinks array
		set nLinks [expr "[array size livegameslinks]/3"]
		set game "Unknown Stream Name"
		for {set i 0} {$i < $nLinks} {incr i} {
			if { $livegameslinks($i,link) == $sopcastlink } {
				set game $livegameslinks($i,game)
			}
		}

		set nextslot [lindex $freechannels 0]
		set port [expr "($nextslot*100)+9000"]
		set vport [expr "$port + 1"]

		set freechannels [lrange $freechannels 1 end]
		.streams.prog$nextslot configure -text $sopcastlink
	
		set fp [open "|sp-sc $sopcastlink $port $vport" r]
		fileevent $fp readable "parseSpSc $fp $nextslot"
		addstream $nextslot $sopcastlink $fp $port $game
	}
}

# kills an open stream specified by stream_no
# currently leaves sp-sc as a zombie process however calling wait
# to destroy the zombie does not currently work.
proc kill_stream {stream_no} {
	global openchannels
	global freechannels
	
	if { [lsearch -exact $freechannels $stream_no] == -1} {
		set fp $openchannels($stream_no,filehandle)	
		set PID [pid $fp]
		fileevent $fp readable ""
		exec kill $PID
		#exec wait
				
		if {[winfo exists .streams]} {
			.streams.prog$stream_no configure -value 0
			.streams.prog$stream_no configure -text "0%"
		}
		
		if {[winfo exists .infostream$stream_no]} {
			.infostream$stream_no.game configure -text "Statistics Unavaliable"
			.infostream$stream_no.link configure -text "Statistics Unavaliable"
			.infostream$stream_no.up configure -text "Statistics Unavaliable"
			.infostream$stream_no.down configure -text "Statistics Unavaliable"
			.infostream$stream_no.peers configure -text "Statistics Unavaliable"
		}
		
		removestream $stream_no
	} else {
		puts "Cant kill stream: $stream_no is not open"
	}
}

# Show the open streams and buffer levels dialog
proc show_streams { } {

	if {![winfo exists .streams]} {
		# open streams window
		toplevel .streams
		wm title .streams "Open Streams"
		wm resizable .streams FALSE FALSE

		# streams GUI elemens
		tixMeter .streams.prog1 -value 0 -width 250
		tixMeter .streams.prog2 -value 0 -width 250
		tixMeter .streams.prog3 -value 0 -width 250
		tixMeter .streams.prog4 -value 0 -width 250

		button .streams.but_play1 -text "Play" -command "push_play 1"
		button .streams.but_play2 -text "Play" -command "push_play 2"
		button .streams.but_play3 -text "Play" -command "push_play 3"
		button .streams.but_play4 -text "Play" -command "push_play 4"
		button .streams.but_kill1 -text "Kill" -command "kill_stream 1"
		button .streams.but_kill2 -text "Kill" -command "kill_stream 2"
		button .streams.but_kill3 -text "Kill" -command "kill_stream 3"
		button .streams.but_kill4 -text "Kill" -command "kill_stream 4"
		button .streams.but_info1 -text "Info" -command "view_stream 1"
		button .streams.but_info2 -text "Info" -command "view_stream 2"
		button .streams.but_info3 -text "Info" -command "view_stream 3"
		button .streams.but_info4 -text "Info" -command "view_stream 4"

		grid .streams.prog1 -row 1 -column 1
		grid .streams.prog2 -row 2 -column 1
		grid .streams.prog3 -row 3 -column 1
		grid .streams.prog4 -row 4 -column 1
		grid .streams.but_play1 -row 1 -column 2
		grid .streams.but_play2 -row 2 -column 2
		grid .streams.but_play3 -row 3 -column 2
		grid .streams.but_play4 -row 4 -column 2
		grid .streams.but_kill1 -row 1 -column 3
		grid .streams.but_kill2 -row 2 -column 3
		grid .streams.but_kill3 -row 3 -column 3
		grid .streams.but_kill4 -row 4 -column 3
		grid .streams.but_info1 -row 1 -column 4
		grid .streams.but_info2 -row 2 -column 4
		grid .streams.but_info3 -row 3 -column 4
		grid .streams.but_info4 -row 4 -column 4
	}
	wm deiconify .streams
}

# display the about dialog.
proc about_clicked { } {
	if {![winfo exists .about]} {
		# open streams window
		toplevel .about
		wm title .about "About"
		wm resizable .about FALSE FALSE
		
		label .about.title -text "SopcastSports v0.1"
		label .about.author -text "Daniel Bardsley"
		label .about.email -text "daniel@bardsley.org.uk"
		label .about.text1 -text "Game data from www.myp2p.eu"
		label .about.text2 -text "Streaming video by sopcast"
		
		pack .about.title -in .about
		pack .about.author -in .about
		pack .about.email -in .about
		pack .about.text1 -in .about
		pack .about.text2 -in .about
	}
}
